from autolens.data.array import grids
from autolens.data import fourier_transform as ft

import numpy as np
import time

repeats = 1

visibilities = int(1e5)
image_pixels = int(1e4)
source_pixels = int(1e3)

shape_data = 8 * visibilities
shape_preloads = visibilities * image_pixels * 2
shape_mapping_matrix = visibilities * source_pixels

total_shape = shape_data + shape_preloads + shape_mapping_matrix

print("Data Memory Use (GB) = " + str(shape_data * 8e-9))
print("PreLoad Memory Use (GB) = " + str(shape_preloads * 8e-9))
print("Mapping Matrix Memory Use (GB) = " + str(shape_mapping_matrix * 8e-9))
print("Total Memory Use (GB) = " + str(total_shape * 8e-9))
print()

uv_wavelengths = np.ones(shape=(visibilities, 2))
grid_radians = np.ones(shape=(image_pixels, 2))
intensities_1d = np.ones(shape=(image_pixels))

transformer = ft.Transformer(uv_wavelengths=uv_wavelengths, grid_radians=grid_radians)


start = time.time()
for i in range(repeats):
    transformer.real_visibilities_from_intensities(intensities_1d=intensities_1d)
diff = time.time() - start
print("Real Visibilities Time = {}".format(diff / repeats))

start = time.time()
for i in range(repeats):
    transformer.real_visibilities_via_preload_from_intensities(intensities_1d=intensities_1d)
diff = time.time() - start
print("Real Visibilities PreLoad Time = {}".format(diff / repeats))